﻿Type=Class
Version=4.01
ModulesStructureVersion=1
B4i=true
@EndOfDesignText@
'version 2.01
#Event: ItemClick (Place As PlaceData)
#DesignerProperty: Key: HighlightColor, DisplayName: Highlight Color, FieldType: Color, DefaultValue: 0xFFFD5C5C
#DesignerProperty: Key: TextColor, DisplayName: Text Color, FieldType: Color, DefaultValue: 0xFF000000
Sub Class_Globals
	Private et As TextField
	Private lv As TableView
	Private mCallback As Object
	Private mEventName As String
	Private mBase As WeakRef
	Private keyboardHeight As Int
	Private QueryIndexSent, QueryIndexReceived As Int
	Private highlightColor As Int = 0xFFFD5C5C
	Private textColor As Int
	Private Const DefaultColorConstant As Int = -984833 'ignore
	Type PlaceData (Description As String, Id As String)
End Sub

'Initializes the object and sets the module and sub that will handle the ItemClick event
Public Sub Initialize (Callback As Object, EventName As String)
	et.Initialize("et")
	et.ShowClearButton = True
	et.BorderStyle = et.STYLE_BEZEL
	lv.Initialize("lv", False)
	lv.Visible = False
	mCallback = Callback
	mEventName = EventName
End Sub

Public Sub DesignerCreateView (Base As Panel, Lbl As Label, Props As Map)
	mBase.Value = Base
	'In this case the views are added to the parent instead of the base panel.
	'This is required as we don't want the base panel to "steal" touch events when the TableView is hidden.
	'UserInteractionEnabled property affects all child views. So we must move the list and text field.
	Base.UserInteractionEnabled = False
	Dim parent As Panel = Base.Parent
    parent.AddView(et, Base.Left, Base.Top, Base.Width, 50dip)
	parent.AddView(lv, Base.Left, Base.Top + et.Height, Base.Width, Base.Height - et.Height)
	highlightColor = Props.Get("HighlightColor")
	textColor = Props.Get("TextColor")
	et.TextColor = textColor
	et_TextChanged("", "")
	et.SetBorder(0, 0, 0)
End Sub

Public Sub KeyboardState(Height As Int)
	keyboardHeight = Height
	Base_Resize(GetBase.Width, GetBase.Height)
End Sub

Private Sub Base_Resize (Width As Double, Height As Double)
	et.SetLayoutAnimated(0, 1, GetBase.Left, GetBase.Top, Width, 50dip)
	lv.SetLayoutAnimated(0, 1, GetBase.Left, GetBase.Top + et.Height, Width, _ 
		Min(GetBase.CalcRelativeKeyboardHeight(keyboardHeight) - et.Height, GetBase.Height - et.Height))
End Sub

Public Sub GetBase As Panel
	Return mBase.Value
End Sub


Private Sub lv_SelectedChanged (SectionIndex As Int, Cell As TableCell)
	Dim pd As PlaceData = Cell.Tag
	et.Text = pd.Description
	lv.Visible = False
	et.ResignFocus
	CallSubDelayed2(mCallback, mEventName & "_ItemClick", pd)
End Sub


Private Sub et_TextChanged (Old As String, New As String)
	If New.Length = 0 Then
		lv.Clear
		lv.Visible = False
	Else
		If lv.Visible = False Then lv.Visible = True
		Dim j As HttpJob
		j.Initialize("", Me)
		j.Download2("https://maps.googleapis.com/maps/api/place/autocomplete/json", _
			Array As String("input", New, "key", Main.GoogleApiKey))
		QueryIndexSent = QueryIndexSent + 1
		Dim index As Int = QueryIndexSent
		Wait For (j) JobDone (j As HttpJob)
		If j.Success Then
			If QueryIndexReceived > index Then
				Return
			Else
				Dim pds As List
				pds.Initialize
				lv.Clear
				QueryIndexReceived = index
				Dim jp As JSONParser
				jp.Initialize(j.GetString)
				Dim m As Map = jp.NextObject
				Dim predictions As List = m.Get("predictions")
				For Each pred As Map In predictions
					Dim pd As PlaceData
					pd.Initialize
					pd.Id = pred.Get("place_id")
					pd.Description = pred.Get("description")
					pds.Add(pd)
				Next
				AddItemsToList(pds, New.ToLowerCase)
			End If
		End If
		j.Release
	End If
End Sub

Private Sub AddItemsToList(li As List, full As String)
	If li.IsInitialized = False Then Return
	Dim cs As CSBuilder
	For Each Item As PlaceData In li
		Dim desc As String = Item.Description
		Dim x As Int = desc.ToLowerCase.IndexOf(full)
		If x = -1 Then
			Continue
		End If
		cs.Initialize.Append(desc.SubString2(0, x)).Color(highlightColor).Append(desc.SubString2(x, x + full.Length)).Pop
		cs.Append(desc.SubString(x + full.Length))
		Dim tc As TableCell = lv.AddSingleLine("")
		tc.Text = cs
		tc.Tag = Item
	Next
	lv.ReloadAll
End Sub


